@@ -127,31 +127,37 @@ def MD_ProbCalc(MD,DF):
 class MyKalmanFilter:
     def __init__(self, TS_Data, SeasonalityPeriods=1):
         OriginalData=TS_Data.data
-        NormalisedData=OriginalData/np.linalg.norm(OriginalData)  
+        NormalisedData=OriginalData/np.linalg.norm(OriginalData) 
+#         NormalisedData=RemoveOutliers(OriginalData)/np.linalg.norm(RemoveOutliers(OriginalData))
         self.OriginalData=OriginalData
         self.data=NormalisedData
         self.dates=TS_Data.dates
         self.SeasonalityPeriods=SeasonalityPeriods
         self.StateArrays=SeasonalStateArrays(SeasonalityPeriods)
+            
         
     def InitialiseFilter(self):
         NormalisedData=self.data
+        MeasureVar=.01   #Doesn't seem to do anything??
         kf_model=KalmanModel(NormalisedData,transition=self.StateArrays.transition,design=self.StateArrays.design)
         
-        start_params = [np.var(NormalisedData)**3, 0.1] #State convariance and observation covariance. These settings will underfit the curve
+        start_params = [np.var(NormalisedData)**3, MeasureVar] #State convariance and observation covariance. These settings will underfit the curve
         kf_model_fit = kf_model.fit(start_params,maxiter = 20,method = 'bfgs', hessian= 'true')
-        Growth=[kf_model_fit.filtered_state[0,x] for x in range(kf_model_fit.filtered_state.shape[1])]
+        Growth=kf_model_fit.predict(1,len(self.data))
         Base=np.mean(abs(NormalisedData-Growth))
 
         Success=False
         i=3
+        OldRatio=0
         while Success ==False and i>0:  #Increae state covariance until fitting improves over underfit base case
-            start_params = [np.var(NormalisedData)**(i), 0.1]
+            start_params = [np.var(NormalisedData)**(i), MeasureVar]
             kf_model_fit = kf_model.fit(start_params,maxiter = 20,method = 'bfgs', hessian= 'true')
-            Growth=[kf_model_fit.filtered_state[0,x] for x in range(kf_model_fit.filtered_state.shape[1])]
-            if np.mean(abs(NormalisedData-Growth))/Base<.3:  
+            Growth=kf_model_fit.predict(1,len(self.data))
+            Ratio=np.mean(abs(NormalisedData-Growth))/Base
+            if Ratio<.3 or Ratio==OldRatio:  
                 Success =True
             i-=0.05
+            OldRatio=Ratio
         
         #Determine kalman gain
 #         K_gain=np.zeros(self.SeasonalityPeriods+1)
@@ -173,11 +179,15 @@ class ChangePointDetectorSession:
         self.TimeseriesModel.InitialiseFilter()
         self.SeasonalityPeriods=SeasonalityPeriods
         self.ForecastPeriods=ForecastPeriods
+        
     
     def ChangePointDetectorFunction(self):
     #This is the main function    
         #Get underlying trend
-        gs_orig=self.TimeseriesModel.KalmanModelFit.filtered_state[-1,:]
+#       gs_orig=self.TimeseriesModel.KalmanModelFit.filtered_state[-1,:]
+        gs_orig=self.TimeseriesModel.KalmanModelFit.smoothed_state[-1,:]
+        gs_orig= np.insert(gs_orig[:-1], 0, 0, axis=0)
+        
         gs=TimeSeriesData(gs_orig,self.ts.dates)
         OnePeriodRegressionModel=MyKalmanFilter(gs,SeasonalityPeriods=0)
         OnePeriodRegressionModel.InitialiseFilter()
@@ -214,14 +224,15 @@ class ChangePointDetectorSession:
         for i in range(self.SeasonalityPeriods+1):
             EVMT.append(0)
         ChangePoints=[]
-        ChangePoint=0
+        ChangePoint=self.SeasonalityPeriods+2
+#         ChangePoint=0
         PrLmOld=0
-        for t in range(self.SeasonalityPeriods+2,len(MDout)+1):
+        for t in range(self.SeasonalityPeriods+3,len(MDout)+1):
             EV=0
             EV_Old=1  #Prl1=1 when t=1
             #Iterate over a window expanding back to the last change point 
-            for m in range(1,t-ChangePoint+1):
-        
+
+            for m in range(1,t-ChangePoint+1):  
                 loc=sqrt(2*log(m+1))-(2*pi+log(log(m+1)))/(2*sqrt(log(m+1)))
                 scale=1/sqrt(2*log(m+1))
                 if m<=2:
@@ -238,18 +249,17 @@ class ChangePointDetectorSession:
                 PrLmOld=PrLm
                 EV_Old=EV
             #Changepoint if likelihood > 95% and not in first few datapoints and no changepoints in last 3 periods
-            if   EV>0.95 and t>SeasonalityPeriods*2 and (t-(ChangePoint+1))>3:  
+            if   EV>0.97 and t>self.SeasonalityPeriods*2+2 and (t-(ChangePoint+1))>2:  
                 EV=1
-                ChangePoint=t-1
+                ChangePoint=t
                 PrLmOld=0
                 EV_Old=1
                 ChangePoints.append(ChangePoint)
-        
             EVMT.append(EV)
             
         ExtendedDates=list(self.ts.dates)
         #Do forecast if no change points in last 3 periods
-        if  not sum([i > len(self.ts.data)-3 for i in ChangePoints]):
+        if  not sum([i > len(self.ts.data)-4 for i in ChangePoints]):
             #Start with historical data
             Prediction=self.TimeseriesModel.KalmanModelFit.predict(0,len(self.ts.data))
             
@@ -296,21 +306,26 @@ class ChangePointDetectorSession:
         Prediction=Prediction*np.linalg.norm(self.ts.data)
         PredictionVariance=PredictionVariance*(np.linalg.norm(self.ts.data)**1)
         
-        Trend=[self.TimeseriesModel.KalmanModelFit.filtered_state[1,x] for x in range(self.TimeseriesModel.KalmanModelFit.filtered_state.shape[1])]
+        Trend=[self.TimeseriesModel.KalmanModelFit.filtered_state[-1,x] \
+               for x in range(self.TimeseriesModel.KalmanModelFit.filtered_state.shape[1])]
         Trend=[x*np.linalg.norm(self.ts.data) for x in Trend]
+        TrendSmoothed=[self.TimeseriesModel.KalmanModelFit.smoothed_state[-1,x] \
+               for x in range(self.TimeseriesModel.KalmanModelFit.filtered_state.shape[1])]
+        TrendSmoothed=[x*np.linalg.norm(self.ts.data) for x in TrendSmoothed]
         
         TrendVariance=[self.TimeseriesModel.KalmanModelFit.filtered_state_cov[-1,-1,x] \
                        for x in range(self.TimeseriesModel.KalmanModelFit.filtered_state_cov.shape[2])]
         TrendVariance=[x*np.linalg.norm(self.ts.data)**1 for x in TrendVariance]
         
-        for i in range(SeasonalityPeriods+2):
+        for i in range(self.SeasonalityPeriods+2):
             PredictionVariance[i]=0
             TrendVariance[i]=0
 
         
-        ChangePoints=[1 if x in ChangePoints else 0 for x in range(len(gs_orig))]
+        ChangePoints=[1 if x in ChangePoints else 0 for x in range(1,len(gs_orig)+1)]
         
-        Results= ModuleResults(Trend=Trend,TrendVariance=TrendVariance,ChangePoints=ChangePoints,Prediction=Prediction,\
+        Results= ModuleResults(Trend=Trend,TrendSmoothed=TrendSmoothed,TrendVariance=TrendVariance,ChangePoints=ChangePoints,Prediction=Prediction,\
                               PredictionVariance=PredictionVariance,ExtendedDates=ExtendedDates)
         
+        
         return Results
\ No newline at end of file
